;*****************************************************************************
;				NICST TOOL
;
;  RC-5 Part 1 test for TEB bands. Calculates the retreived EV source radiance
;  and compares to specifications. Also, the detector uniformity is investigated.
;
;  Input:
;  1) Input file with paths to RVS, and general info sav files as well
;     as the .temp.sav, .dn.sav, .rad.sav, .rg.sav, and .snr.sav files.
;
;  Output:
;  1) rad_BCS_ret[collect,band,HAM,detector,subsample,ifit,rsr]
;  2) ARD[collect,band,HAM,detector,subsample,ifit,rsr]
;  3) RRU[collect,band,HAM,subsample,ifit,NEDL at spec or measured NEdL,rsr]
;  4) RRU_det[collect,band,HAM,detector,subsample,ifit,NEDL at spec or measured NEdL,rsr]
;  5) mean_ARD[collect,band,HAM,subsample,ifit,rsr]
;  6) max_ARD[collect,band,HAM,subsample,ifit,rsr]
;  7) ARD_gc[collect,band,HAM,detector,subsample,ifit,rsr] -- ARD minus fitting residual
;  8) mean_ARD_gc[collect,band,HAM,subsample,ifit,rsr] -- mean ARD over detectors minus fitting residual
;  
;  v1.0 - 04/14/2008 - Jeff McIntire
;  v2.0 - 03/23/2009 - Jeff McIntire
;   -- Added RRU per detector.
;*****************************************************************************

PRO RC5_eff_rad, infile

;*****************************************************************************
;  Read in input file and sav TEB file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
Free_Lun,lun

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

restore,'general.TEB.2.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.snr.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.eff_rad.sav'

;*****************************************************************************
;  Definitions
;*****************************************************************************

NEDT_spec=[[2.5,1.5,0.396,0.107,0.091,0.07,0.072,0.072],[0,0,0,0.423,0,0,0,0]]

mindet=[0,0,0,0,1,0,0,0]

fit_res=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,3,2)

NEdL_det=dblarr(nbands,nham,maxdet,maxsubsets,2,2,2)

rad_BCS_ret=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,3,2)
delta_rad_BCS_ret=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,3,2)
mean_rad_BCS_ret=dblarr(ncollects,nbands,nham,maxsubsets,3,2)
max_rad_BCS_ret=dblarr(ncollects,nbands,nham,maxsubsets,3,2)
min_rad_BCS_ret=dblarr(ncollects,nbands,nham,maxsubsets,3,2)
std_rad_BCS_ret=dblarr(ncollects,nbands,nham,maxsubsets,3,2)

ARD=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,3,2)
mean_ARD=dblarr(ncollects,nbands,nham,maxsubsets,3,2)
max_ARD=dblarr(ncollects,nbands,nham,maxsubsets,3,2)
std_ARD=dblarr(ncollects,nbands,nham,maxsubsets,3,2)
ARD_gc=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,3,2)
mean_ARD_gc=dblarr(ncollects,nbands,nham,maxsubsets,3,2)

RRU=dblarr(ncollects,nbands,nham,maxsubsets,3,2,2)
RRU_det=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,3,2,2)

;*****************************************************************************
;  Find retrived BCS radiance for RC5 part 1.
; ifit 0-2 for linear, quadratic, and cubic with gain correction.
; For each fit, the fractional residual in percent is calculated.
;*****************************************************************************

FOR rsr=0, 1 DO BEGIN
    FOR ifit=0, 2 DO BEGIN
        FOR col=0, ncollects-1 DO BEGIN
            FOR b=0, nbands-1 DO BEGIN
                FOR is=0, subsets[b]-1 DO BEGIN
                    FOR ih=0, nham-1 DO BEGIN
                        FOR id=0, detectors[b]-1 DO BEGIN

                            ; Ensure the proper subset is used for the OBC dn.
                            IF dn[col,b,ih,id,0,1,0] GT dn[col,b,ih,id,1,1,0] AND dn_obc[col,b,ih,id,0,1,0] GT dn_obc[col,b,ih,id,1,1,0] THEN BEGIN 
                                isubset=is
                            ENDIF ELSE BEGIN
                                IF is EQ 0 THEN isubset=1
                                IF is EQ 1 THEN isubset=0
                            ENDELSE

                            IF ifit EQ 0 THEN BEGIN
                                rad_BCS_ret[col,b,ih,id,is,ifit,rsr] $
                                  =(rg_1[b,ih,id,is,0,0,rsr]+rg_1[b,ih,id,is,1,0,rsr]*dn[col,b,ih,id,is,1,0])/rvs[b,is,id,ih,0]/p[b] $
                                  *(sig_mean(rad_OBC[col,b,ih,id,is,0:99,rsr],3,1)/(rg_1[b,ih,id,is,0,0,rsr]+rg_1[b,ih,id,is,1,0,rsr]*dn_obc[col,b,ih,id,isubset,1,0])) $
                                  +(rvs[b,is,id,ih,1]/rvs[b,is,id,ih,0])*rad_p1[col,b,id,1,rsr] $
                                  -(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,0])/(rvs[b,is,id,ih,0]*p[b])*(rad_p2[col,b,ih,id,0,rsr]-(1-p[b])*rad_p2[col,b,ih,id,1,rsr])
                                fit_res[col,b,ih,id,is,ifit] $
                                  =(rad_BCS[col,b,ih,id,is,rsr]*0.4444-rg_1[b,ih,id,is,0,0,rsr]-rg_1[b,ih,id,is,1,0,rsr]*dn[col,b,ih,id,is,1,0]) $
                                  /(rad_BCS[col,b,ih,id,is,rsr]*0.4444)*100
                            ENDIF
                            IF ifit EQ 1 THEN BEGIN
                                rad_BCS_ret[col,b,ih,id,is,ifit,rsr] $
                                  =(rg_1[b,ih,id,is,0,1,rsr]+rg_1[b,ih,id,is,1,1,rsr]*dn[col,b,ih,id,is,1,0]+rg_1[b,ih,id,is,2,1,rsr]*dn[col,b,ih,id,is,1,0]^2) $
                                  /(rvs[b,is,id,ih,0]*p[b]) $
                                  *(sig_mean(rad_OBC[col,b,ih,id,is,0:99,rsr],3,1)/(rg_1[b,ih,id,is,0,1,rsr]+rg_1[b,ih,id,is,1,1,rsr]*dn_obc[col,b,ih,id,isubset,1,0] $
                                                                                +rg_1[b,ih,id,is,2,1,rsr]*dn_obc[col,b,ih,id,isubset,1,0]^2))  $
                                  +(rvs[b,is,id,ih,1]/rvs[b,is,id,ih,0])*rad_p1[col,b,id,1,rsr] $
                                  -(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,0])/(rvs[b,is,id,ih,0]*p[b])*(rad_p2[col,b,ih,id,0,rsr]-(1-p[b])*rad_p2[col,b,ih,id,1,rsr])
                                fit_res[col,b,ih,id,is,ifit] $
                                  =(rad_BCS[col,b,ih,id,is,rsr]*0.4444-rg_1[b,ih,id,is,0,1,rsr]-rg_1[b,ih,id,is,1,1,rsr]*dn[col,b,ih,id,is,1,0] $
                                    -rg_1[b,ih,id,is,2,1,rsr]*dn[col,b,ih,id,is,1,0]^2)/(rad_BCS[col,b,ih,id,is,rsr]*0.4444)*100
                            ENDIF
                            IF ifit EQ 2 THEN BEGIN 
                                rad_BCS_ret[col,b,ih,id,is,ifit,rsr] $
                                  =(rg_1[b,ih,id,is,0,2,rsr]+rg_1[b,ih,id,is,1,2,rsr]*dn[col,b,ih,id,is,1,0]+rg_1[b,ih,id,is,2,2,rsr]*dn[col,b,ih,id,is,1,0]^2 $
                                    +rg_1[b,ih,id,is,3,2,rsr]*dn[col,b,ih,id,is,1,0]^3)/rvs[b,is,id,ih,0]/p[b] $
                                  *(sig_mean(rad_OBC[col,b,ih,id,is,0:99,rsr],3,1)/(rg_1[b,ih,id,is,0,2,rsr]+rg_1[b,ih,id,is,1,2,rsr]*dn_obc[col,b,ih,id,isubset,1,0] $
                                                                                +rg_1[b,ih,id,is,2,2,rsr]*dn_obc[col,b,ih,id,isubset,1,0]^2 $
                                                                                +rg_1[b,ih,id,is,3,2,rsr]*dn_obc[col,b,ih,id,isubset,1,0]^3)) $
                                  +(rvs[b,is,id,ih,1]/rvs[b,is,id,ih,0])*rad_p1[col,b,id,1,rsr] $
                                  -(rvs[b,is,id,ih,1]-rvs[b,is,id,ih,0])/(rvs[b,is,id,ih,0]*p[b])*(rad_p2[col,b,ih,id,0,rsr]-(1-p[b])*rad_p2[col,b,ih,id,1,rsr])
                                fit_res[col,b,ih,id,is,ifit] $
                                  =(rad_BCS[col,b,ih,id,is,rsr]*0.4444-rg_1[b,ih,id,is,0,2,rsr]-rg_1[b,ih,id,is,1,2,rsr]*dn[col,b,ih,id,is,1,0] $
                                    -rg_1[b,ih,id,is,2,2,rsr]*dn[col,b,ih,id,is,1,0]^2-rg_1[b,ih,id,is,3,2,rsr]*dn[col,b,ih,id,is,1,0]^3) $
                                  /(rad_BCS[col,b,ih,id,is,rsr]*0.4444)*100
                            ENDIF
                            
;*****************************************************************************
;  Caclulate the ARD.
;*****************************************************************************

                            ARD[col,b,ih,id,is,ifit,rsr]=100*(rad_BCS_ret[col,b,ih,id,is,ifit,rsr]/rad_p1[col,b,id,0,rsr]-1.)
                            ARD_gc[col,b,ih,id,is,ifit,rsr]=ARD[col,b,ih,id,is,ifit,rsr]+fit_res[col,b,ih,id,is,ifit,rsr]
                            
                            delta_rad_BCS_ret[col,b,ih,id,is,ifit,rsr]=rad_BCS_ret[col,b,ih,id,is,ifit,rsr]-rad_p1[col,b,id,0,rsr]
                            NEdL_det[b,ih,id,is,0,1,rsr]=rad_BCS[col,b,ih,id,is,rsr]*0.4444/SNR[col,b,ih,id,is,1]
                        ENDFOR

                        mean_rad_BCS_ret[col,b,ih,is,ifit,rsr]=mean(delta_rad_BCS_ret[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr])
                        max_rad_BCS_ret[col,b,ih,is,ifit,rsr]=max(delta_rad_BCS_ret[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr],min=min_rad)
                        min_rad_BCS_ret[col,b,ih,is,ifit,rsr]=min_rad
                        std_rad_BCS_ret[col,b,ih,is,ifit,rsr]=sig_mean_no_kick(delta_rad_BCS_ret[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr],2)

                        mean_ARD[col,b,ih,is,ifit,rsr]=mean(ARD[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr])
                        mean_ARD_gc[col,b,ih,is,ifit,rsr]=mean(ARD_gc[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr])
                        max_ARD[col,b,ih,is,ifit,rsr]=max(abs(ARD[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr]))
                        std_ARD[col,b,ih,is,ifit,rsr]=sig_mean_no_kick(ARD[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr],2)
                                        
;*****************************************************************************
;  Calculate the RRU using both retrieved and delta retrieved as
;  well as both NEdL at spec and measured NEdL
;*****************************************************************************

                        ; Calculate the RRU per detector.
                        temp=mean(delta_rad_BCS_ret[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr])
                        temp1=mean(rad_BCS_ret[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,rsr])
                        FOR id=0, detectors[b]-1 DO BEGIN
                            RRU_det[col,b,ih,id,is,ifit,0,rsr]=0.4444*abs(delta_rad_BCS_ret[col,b,ih,id,is,ifit,rsr]-temp) $
                              /(NEdT_spec[b,0]*Planck_deriv(T_typ[b,0],lambda_center[b]))
                       ;     RRU_det[col,b,ih,id,is,ifit,0,rsr]=0.4444*abs(rad_BCS_ret[col,b,ih,id,is,ifit,rsr]-temp1) $
                       ;       /(NEdT_spec[b,0]*Planck_deriv(T_typ[b,0],lambda_center[b]))
                            RRU_det[col,b,ih,id,is,ifit,1,rsr]=0.4444*abs(delta_rad_BCS_ret[col,b,ih,id,is,ifit,rsr]-temp) $
                              /NEdL_det[b,ih,id,is,0,1,rsr]
                        ENDFOR
                        RRU[col,b,ih,is,ifit,0,rsr]=max(RRU_det[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,0,rsr])
                        RRU[col,b,ih,is,ifit,1,rsr]=max(RRU_det[col,b,ih,mindet[b]:detectors[b]-1,is,ifit,1,rsr])
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Save data on ARD and RRU to files.
;*****************************************************************************

outfile1=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.ard.txt'
outfile2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rru.txt'

fmt_1='(i2, 5x, i3, 5x, i2, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f, 5x, f)'

openw,lun,outfile1,/get_lun
printf,lun,'Mean Absolute Radiance Difference (ARD) with gain correction'
printf,lun,'Collect     Temp       HAM      I4        I5       M12       M13        M14      M15      M16A      M16B'
FOR ih=0, nham-1 DO BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        printf,lun,format=fmt_1,col,T_bcs[col],ih,mean(mean_ARD[col,0,ih,*,1,0]),mean(mean_ARD[col,1,ih,*,1,0]), $
          mean_ARD[col,2,ih,0,1,0],mean_ARD[col,3,ih,0,1,0],mean_ARD[col,4,ih,0,1,0], $
          mean_ARD[col,5,ih,0,1,0],mean_ARD[col,6,ih,0,1,0],mean_ARD[col,7,ih,0,1,0]
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile2,/get_lun
printf,lun,'Worst case Relative Response Uniformity (RRU) with gain correction'
printf,lun,'Collect     Temp       HAM      I4        I5       M12       M13        M14      M15      M16A      M16B'
FOR ih=0, nham-1 DO BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        printf,lun,format=fmt_1,col,T_bcs[col],ih,mean(RRU[col,0,ih,*,1,0,0]),mean(RRU[col,1,ih,*,1,0,0]), $
          RRU[col,2,ih,0,1,0,0],RRU[col,3,ih,0,1,0,0],RRU[col,4,ih,0,1,0,0], $
          RRU[col,5,ih,0,1,0,0],RRU[col,6,ih,0,1,0,0],RRU[col,7,ih,0,1,0,0]
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

save,/compress,rad_BCS_ret,ARD,ARD_gc,max_ARD,mean_ARD,mean_ARD_gc,RRU,RRU_det,filename=outfile

;*****************************************************************************
;  Plot the ARD.
;*****************************************************************************

set_multicolor,xsave,ysave,psave,chname,lcol,symbol,red,green,blue
set_plot,'ps'
outfname=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.ard.ps'
device,/landscape,filename=outfname,/color
!p.multi=[0,1,1]
!p.position=0

ARD_req_I=[5,2.5]
ARD_req_M=[[0,0,12.3,2.1,1.6,1.6],[7.0,5.7,2.4,0.60,.60,.60],[0.7,0.7,0.6,0.4,0.4,0.4], $
	[0.7,0.7,0.4,0.4,0.4,0.4],[0.7,0.7,0.5,0.4,0.4,0.4]]
ARD_req_temp=[190.,230.,270.,310.,340.]

G=findgen(30)
FOR b=0, nbands-1 DO BEGIN
    title1='Absolute Radiance Difference, (SRV0545-6)'
    title2=band_name[b]+' (quadratic poly_fit)'

    ymax=max(ARD[0:ncollects-1,b,0,0:detectors[b]-1,0,1,0],min=ymin)

    plot,T_bcs[0:ncollects-1],ARD[0:ncollects-1,b,0,0,0,1,0],xtit='BCS Temperature (K)',ytit='ARD',yrange=[-1,1], $
      xrange=[190.,350.],title=title2,charsize=1.2,charthick=5,thick=3,/nodata

    FOR id=0,detectors[b]-1 DO BEGIN
        oplot,T_bcs[0:ncollects-1],ARD[0:ncollects-1,b,0,id,0,1,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
        oplot,T_bcs[0:ncollects-1],ARD[0:ncollects-1,b,0,id,0,1,0],color=lcol[id],thick=3
    ENDFOR
    IF band_type[b] NE 1 AND b GE 4 THEN oplot,ARD_req_temp[0:4],ARD_req_M[b-2,0:4],color=lcol[1],linestyle=2,thick=3
    IF band_type[b] NE 1 AND b GE 4 THEN oplot,ARD_req_temp[0:4],-ARD_req_M[b-2,0:4],color=lcol[1],linestyle=2,thick=3
    IF band_type[b] NE 1 AND b LT 4 THEN oplot,ARD_req_temp[1:4],ARD_req_M[b-2,1:4],color=lcol[1],linestyle=2,thick=3
    IF band_type[b] NE 1 AND b LT 4 THEN oplot,ARD_req_temp[1:4],-ARD_req_M[b-2,1:4],color=lcol[1],linestyle=2,thick=3

    XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
ENDFOR

plot,T_bcs[0:ncollects-1],mean_ARD[0:ncollects-1,0,0,0,1,0],xtit='BCS Temperature (K)',ytit='ARD',yrange=[-0.5,1], $
  xrange=[190.,350.],charsize=1.2,charthick=5,thick=3,/nodata
FOR b=0, nbands-1 DO BEGIN
    oplot,T_bcs[0:ncollects-1],mean_ARD[0:ncollects-1,b,0,0,1,0],symsize=1,color=lcol[b],psym=symbol[b],thick=3
    oplot,T_bcs[0:ncollects-1],mean_ARD[0:ncollects-1,b,0,0,1,0],color=lcol[b],thick=3
ENDFOR
XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
set_legend,1,band_name,lcol,symbol

;*****************************************************************************
;  Plot the RRU.
;*****************************************************************************

title1='Relative Response Uniformity, (SRV0613)'

; 0.9 Tmax
;T_max_9=[317.7,306.,317.7,308.7,302.4,308.7,306.,306.]

; 0.9 Lmax
T_max_9=[349.6,330.8,349.6,339.5,329.2,334.1,330.4,330.4]

plot,T_bcs[0:ncollects-1],RRU[0:ncollects-1,0,0,0,1,0,0],xtit='BCS Temperature (K)',ytit='RRU', $
  yrange=[0,1.0],xrange=[230.,350.],charsize=1.2,charthick=5,thick=3,/nodata
FOR b=0, nbands-1 DO BEGIN
    zz=WHERE(T_bcs[*] GT T_min[b,0] AND T_bcs[*] LT T_max_9[b],arr1)
    oplot,T_bcs[zz],RRU[zz,b,0,0,1,0,0],symsize=1,color=lcol[b],psym=symbol[b],thick=3
    oplot,T_bcs[zz],RRU[zz,b,0,0,1,0,0],color=lcol[b],thick=3
ENDFOR
XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
set_legend,1,band_name,lcol,symbol

FOR b=0, nbands-1 DO BEGIN
    zz=WHERE(T_bcs[*] GT T_min[b,0] AND T_bcs[*] LT T_max_9[b],arr1)
    IF b EQ 4 THEN zz=WHERE(T_bcs[*] GT 200. AND T_bcs[*] LT T_max_9[b],arr1)

    plot,T_bcs[zz],RRU_det[zz,b,0,0,0,1,0,0],xtit='BCS Temperature (K)',ytit='RRU', $
      yrange=[0,max(RRU_det[zz,b,0,mindet[b]:detectors[b]-1,0,1,0,0])],xrange=[190.,350.],title=band_name[b],charsize=1.2,charthick=5,thick=3,/nodata

    FOR id=mindet[b], detectors[b]-1 DO BEGIN
        oplot,T_bcs[zz],RRU_det[zz,b,0,id,0,1,0,0],symsize=1,color=lcol[id],psym=symbol[id],thick=3
        oplot,T_bcs[zz],RRU_det[zz,b,0,id,0,1,0,0],color=lcol[id],thick=3
    ENDFOR
    XYOUTS,0.5,1.025,title1,/normal,alignment=0.5,charsize=1.6,charthick=5
    IF band_type[b] EQ 0 THEN set_legend,1,chname,lcol,symbol
    IF band_type[b] EQ 1 THEN set_legend,2,chname,lcol,symbol
ENDFOR

device,/close

END
